/*
 * kylin-status-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangguixi <wangguixi@kylinos.cn>
 *
 */
#include "rotationmonitoring.h"
#include <QDebug>
#include <QProcess>
#include <QCoreApplication>
#include <QTime>
#define SETTINGS_KYLINSTATUSMANAGER    "org.kylin-status-manager"
#define AUTO_ROTATION        "autorotation" // 旋转
#define ROTATIONS            "rotations"
#define FIRSTSTARTROTATIONS   "firststartrotation"
RotationMonitoring::RotationMonitoring(QObject *parent) : QObject(parent)
{
    QByteArray id(SETTINGS_KYLINSTATUSMANAGER);
    if(QGSettings::isSchemaInstalled(id)) {
        ksmSetting = new QGSettings(id);
        autoRotation = ksmSetting->get(AUTO_ROTATION).toBool();
        connect(ksmSetting, &QGSettings::changed, [this](const QString & key) {
            qInfo() << __FILE__ << __LINE__ << "【org.kylin-status-manager】changed key is：" << key;
            if (key == AUTO_ROTATION) {
                autoRotation = ksmSetting->get(AUTO_ROTATION).toBool();
            }
        });
        qDebug() << __FILE__ << __LINE__ << "【sensor support？】";
        QList<QByteArray> sensorTypes = QSensor::sensorTypes();
        qDebug() << __FILE__ << __LINE__ << "支持的传感器有 ：" << sensorTypes;
        if(sensorTypes.contains("QOrientationSensor")) {
            qInfo() << __FILE__ << __LINE__ << "支持角度传感器";
            isSupported();
            startSensorMonitoring();
        }else {
            qInfo() << __FILE__ << __LINE__ << "不支持角度传感器";
            isNotSupported();
        }
    }else {
        qWarning() << __FILE__ << __LINE__ << "org.kylin-status-manager is not install";
        return;
    }
    if(ksmSetting->get(FIRSTSTARTROTATIONS).toBool()) {
        if(!connectSensor) {
            qInfo() << __FILE__ << __LINE__ << "第一次启动未发现角度传感器，自动旋转设置为false";
            ksmSetting->set(AUTO_ROTATION,false);
        }
        ksmSetting->set(FIRSTSTARTROTATIONS,false);

    }
}
RotationMonitoring::~RotationMonitoring()
{
    if(qSensor) {
        delete qSensor;
    }
    if(ksmSetting) {
        delete ksmSetting;
    }
}
void RotationMonitoring::startSensorMonitoring()
{
    qSensor = new QOrientationSensor();
    connect(qSensor, &QOrientationSensor::readingChanged, this, &RotationMonitoring::rotationsChange);
    if(qSensor->start()) {
        qInfo() << __FILE__ << __LINE__ << "qSensor start successful";
    }else {
        qWarning() << __FILE__ << __LINE__ << "qSensor start failt";
        isNotSupported();
    }
}
void RotationMonitoring::rotationsChange()
{
    bool openRotationsMonitor = autoRotation;
    qDebug() << __FILE__ << __LINE__ << "key is :autoRotation" << openRotationsMonitor;
    if(!openRotationsMonitor) {//自动旋转打开
        return;
    }
    if(!qSensor->isActive()) {
        qWarning() << __FILE__ << __LINE__ << "qSensor is not Active";
        return ;
    }
    int valueCount = qSensor->reading()->valueCount();
    if(valueCount == 0) {
        return ;
    }

    int statusnum = qSensor->reading()->value(0).toInt();
    QString rotations = "normal";
    switch (statusnum) {
    case NORMAL:
        rotations = "normal";
        break;
    case UPSIDEDOWM:
        rotations = "upside-down";
        break;
    case LEFT:
        rotations = "left";
        break;
    case RIGHT:
        rotations = "right";
        break;
    case UNDEFINED:
        qWarning() << "!!!获取的传感器值异常";
        rotations = "undefined";
        break;
    default:
        break;
    }
    qDebug() << __FILE__ << __LINE__ << "sensor return num is :" << statusnum << ",rotations status is :" << rotations;
    Q_EMIT rotationsChangeSignal(rotations);

}
QString RotationMonitoring::getRotationStatus()
{
    qDebug() << __FILE__ << __LINE__ ;
    QString rotations = "normal";
    if(!qSensor->isActive()) {
        qWarning() << __FILE__ << __LINE__ << "qSensor is not Active";
        return rotations;
    }
    int valueCount = qSensor->reading()->valueCount();
    if(valueCount == 0){
        qWarning() << __FILE__ << __LINE__ << "reading valueCount fail";
        return rotations;
    }
    int statusnum = qSensor->reading()->value(0).toInt();
    qDebug() << __FILE__ << __LINE__ << statusnum;
    switch (statusnum) {
    case NORMAL:
        rotations = "normal";
        break;
    case UPSIDEDOWM:
        rotations = "upside-down";
        break;
    case LEFT:
        rotations = "left";
        break;
    case RIGHT:
        rotations = "right";
        break;
    case UNDEFINED:
        qWarning() << "!!!获取的传感器值异常";
        rotations = "undefined";
        break;
    default:
        break;
    }
    return rotations;
}
bool RotationMonitoring::isConnectOrientationSensor()
{
    return connectSensor;
}
void RotationMonitoring::isNotSupported()
{
    connectSensor = false;
}
void RotationMonitoring::isSupported()
{
    connectSensor = true;
}
