/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with thi1s program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "tabletmodeswitchmonitoring.h"
#include "libinputswitchevent.h"

#define SETTINGS_KYLINSTATUSMANAGER    "org.kylin-status-manager"
#define MODE_MONITER         "modemonitor" // 是否监听角度值
#define FRIST_START  "firststart" //第一次启动
TabletModeSwitchMonitoring::TabletModeSwitchMonitoring(QObject *parent) : QObject(parent)
{
    QByteArray id(SETTINGS_KYLINSTATUSMANAGER);
    if(QGSettings::isSchemaInstalled(id)) {
        ksmSetting = new QGSettings(id);
        modemonitor = ksmSetting->get(MODE_MONITER).toBool();
        connect(ksmSetting, &QGSettings::changed, [this](const QString & key) {
            qInfo() << __FILE__ << __LINE__ << "【org.kylin-status-manager】changed key is：" << key;
            if (key == MODE_MONITER) {
                modemonitor = ksmSetting->get(MODE_MONITER).toBool();
            }
        });
    }else {
        qWarning() << __FILE__ << __LINE__ << "org.kylin-status-manager is not install";
        return;
    }

    qDebug() << __FILE__ << __LINE__ << "create libinput switch event";
    libswitch = new LibinputSwitchEvent;
    if(!libswitch) {
        qWarning() << __FILE__ << __LINE__ << "libswitch is nullptr";
        ksmSetting->set(MODE_MONITER,false);
        return;
    }
    //第一次启动时，如果geInitDevicesStatus获取的状态为-1,则设置自动切换为false
    if(ksmSetting->get(FRIST_START).toBool()) {
        if(libswitch->geInitDevicesStatus() == -1) {
            qInfo() << __FILE__ << __LINE__ << "Switch mode status device is not exist";
            //初始化gsetting设置为关闭
            ksmSetting->set(MODE_MONITER,false);
        }
        ksmSetting->set(FRIST_START,false);
    }

    connect(libswitch , &LibinputSwitchEvent::tabletModeStatusChanged, this, [ = ](int tablet_mode) {
        qDebug() << __FILE__ << __LINE__ << "libinput switch event changed modemonitor" << modemonitor;
        qInfo() << __FILE__ << __LINE__ << "tablet mode changed" << "tablet mode is :" << tablet_mode;
        if(tablet_mode == 1) {
            if(modemonitor) {
                Q_EMIT tabletModeChangeSignal(true);
            }
            Q_EMIT builtinKeyboardChangeSignal(false);
        }else {
            if(modemonitor) {
                Q_EMIT tabletModeChangeSignal(false);
            }
            Q_EMIT builtinKeyboardChangeSignal(true);
        }

    });
}
TabletModeSwitchMonitoring::~TabletModeSwitchMonitoring()
{
    if(libswitch){
        delete libswitch;
    }
    if(ksmSetting){
        delete ksmSetting;
    }
}
bool TabletModeSwitchMonitoring::getTabletModeStatus()
{

    if(libswitch) {
        int status = libswitch->geInitDevicesStatus();
        qDebug() << __FILE__ << __LINE__ << "geInitDevicesStatus status is: " << status;
        if(status == 1){
            return true;
        }else if(status == 0){
            return false;
        }else {
            //返回值异常，不支持平板模式
            qWarning() << __FILE__ << __LINE__ << "geInitDevicesStatus 获取设备状态异常，返回值为-1";
            return false;
        }
    }else {
        return false;
    }
}
bool TabletModeSwitchMonitoring::getBuiltinKeyboardStatus()
{
    if(libswitch) {
        int status = libswitch->geInitDevicesStatus();
        if(status == 1){
            return false;
        }else if(status == 0){
            return true;
        }else {
            //返回值异常，不支持平板模式
            qWarning() << __FILE__ << __LINE__ << "geInitDevicesStatus 获取设备状态异常，返回值为-1";
            return true;
        }
    }else {
        return true;
    }
}
