/*
 * kylin-status-manager
 *
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: wangguixi <wangguixi@kylinos.cn>
 *
 */
#include "devicesstatusmonitoring.h"

DevicesStatusMonitoring::DevicesStatusMonitoring(QObject *parent) : QObject(parent)
{

    m_modeStatus = new TabletModeSwitchMonitoring;
    connect(m_modeStatus, &TabletModeSwitchMonitoring::tabletModeChangeSignal,
            this, &DevicesStatusMonitoring::modeChangeSignal);
    connect(m_modeStatus, &TabletModeSwitchMonitoring::builtinKeyboardChangeSignal,
            this, &DevicesStatusMonitoring::keyboardEnableSignal);


    m_rotationStatus = new RotationMonitoring();//旋转监控
    //是否连接方向传感器（即：设备是否支持方向传感器）
    if(m_rotationStatus->isConnectOrientationSensor()) {
        orientationSensor = true;
        connect(m_rotationStatus, &RotationMonitoring::rotationsChangeSignal,
                this, &DevicesStatusMonitoring::rotationsChangeSignal);
    }
//+旋转防抖处理
}
DevicesStatusMonitoring::~DevicesStatusMonitoring()
{
    delete m_rotationStatus;
    delete m_modeStatus;
}
bool DevicesStatusMonitoring::getKeyboardEnableResult()
{
    return m_modeStatus->getBuiltinKeyboardStatus();
}
QString DevicesStatusMonitoring::getRotationStatus()
{
    if(orientationSensor) {
        return m_rotationStatus->getRotationStatus();
    }else {
        return "normal";
    }
}
bool DevicesStatusMonitoring::getTabletModeStatus()
{
    return m_modeStatus->getTabletModeStatus();
}
bool DevicesStatusMonitoring::getOrientationSensor()
{
    return orientationSensor;
}
