/*
 * kylin-status-manager
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef AUTOSWITCH_H
#define AUTOSWITCH_H

#include <QObject>
#include <QTimer>
#include <QTime>
#include <QSensor>
#include <QGSettings>
#include "Devices_Status_Monitoring/devicesstatusmonitoring.h"

class QTimer;
class QDBusInterface;

class AutoSwitch : public QObject
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.kylin.statusmanager.interface")

public:
    explicit AutoSwitch(QObject *parent = nullptr);
    ~AutoSwitch();

    enum FoldStatus
    {
        NORMAL = 1,    //水平向下，不改变pc/table模式，不改变前一个信号值
        UPSIDDOWN = 2,  //
        LEFT = 3,        //
        RIGHT = 4 ,      //
    };
    Q_ENUM(FoldStatus)
public slots:
    int get_inputmethod_mode();                                           //获得当前键盘模式
    QString get_current_stylename();                                      //获得当前主题名字
    QString get_current_rotation();                                       //获得当前旋转情况
    bool get_current_tabletmode();                                        //获得当前的模式
    bool get_modemonitor();
    bool get_auto_rotation();
    bool is_supported_autorotation();
    bool set_rotation(QString rotation,QString who,QString why);
    bool set_tabletmode(bool tabletMode,QString who,QString why);
    bool set_modemonitor(bool modeMonitor,QString who,QString why);
    bool set_auto_rotation(bool autoRotation,QString who,QString why);

private:
    struct ksmData
    {
        QString rotation;
        bool tabletMode;
        bool modeMonitor;
        bool autoRotation;
        bool keyboardEnable;
        bool orientationSensor;
    };
    ksmData ksmstruct;
    //QGSettings
    QGSettings *themeSetting = nullptr; //主题gsetting
    QGSettings *ksmSetting = nullptr;   //状态管理器的gsetting
    //防抖
//    QTimer *debounceTimer = Q_NULLPTR; //计时器 、fold
    QTimer *debounceTimer1 = Q_NULLPTR; //计时器、
    QTimer *debounceTimer2 = Q_NULLPTR; //计时器、键盘
    int foldtime = 0;
    int rotetime = 0;
    int keyboardtim = 0;
    //设备检测
    DevicesStatusMonitoring * devicestatus = nullptr;
    bool m_keyboardstatus = true;
    void init();
    void registDbus();
    void updateKsmstruct();
    void updateModeStatus(bool tabletMode);
    void updateRotationStatus();
    void updateKeyboardStatus();//更新键盘状态 //改为根据当前键盘情况更新状态 发送信号
    bool setKeyboardEnable(bool keyboardEnable,QString who,QString why);


    //新需求：装完系统第一次切换平板模式时调出用户手册-触控手势介绍页
    bool openUserGuide();
signals:
    void inputmethod_change_signal(bool keyboardEnabled);                // 获取电脑状态 获取键盘禁用状态;false:禁用;
    void stylename_change_signal(QString styleName);                     // 主题更改信号
    void rotations_change_signal(QString rotations);                      // 旋转改变信号
    void mode_change_signal(bool tabletMode);                            // 模式改变信号
    void modemonitor_change_signal(bool modeMonitor);                     // 模式监控改变信号
    void auto_rotation_change_signal(bool autoRotation);                 // 自动旋转改变信号

    void showUserGuideSignal();
private slots:

    void modeDebounceTimeout();//防抖翻折
    void rotationsDebounceTimeout();//防抖陀螺仪
    void keyboardDebounceTimeout();//防抖陀螺仪

    void modeDebounce(int num);
    void rotationsDebounce(int num);
    void keyboardDebounce(int num);

    void processingKeyboardEnableSignal(bool enable);
    void processingRotationsChangeSignal(QString rotations);
    void processingModeChangeSignal(bool tabletMode);

};

#endif // AUTOSWITCH_H
